"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = exports.resolve = void 0;
var path_1 = __importDefault(require("path"));
var utils_1 = require("./utils");
var is_module_import = function (filepath, options) {
    var basedir = options.basedir;
    try {
        require.resolve(filepath, { paths: [basedir] });
    }
    catch (e) {
        return false;
    }
    return true;
};
var is_already_specific = function (filepath, options) {
    var platform_extensions = options.platform_extensions, extensions = options.extensions;
    var extensionless = (0, utils_1.is_extensionless)(filepath, extensions);
    var parsed = path_1.default.parse(filepath);
    var name = parsed.name;
    var ext = parsed.ext;
    if (extensionless) {
        /**
         * filepath = 'example_file.win' // e.g. "extensionless"
         * -> parsed.name = example_file
         * -> parsed.ext = win
         */
        if (parsed.ext !== '') {
            name += ".".concat(parsed.ext);
        }
        ext = '';
    }
    var ends_with_platform = false;
    for (var _i = 0, platform_extensions_1 = platform_extensions; _i < platform_extensions_1.length; _i++) {
        var platform_extension = platform_extensions_1[_i];
        if (name.endsWith(".".concat(platform_extension))) {
            ends_with_platform = true;
            break;
        }
    }
    return ends_with_platform && (ext === '' || extensions.includes(ext));
};
var try_resolve = function (filepath, options) {
    var platform_extensions = options.platform_extensions, extensions = options.extensions, basedir = options.basedir;
    var filepath_stripped = (0, utils_1.strip_extension)(filepath, extensions, platform_extensions);
    for (var _i = 0, platform_extensions_2 = platform_extensions; _i < platform_extensions_2.length; _i++) {
        var platform_extension = platform_extensions_2[_i];
        for (var _a = 0, extensions_1 = extensions; _a < extensions_1.length; _a++) {
            var extension = extensions_1[_a];
            var alternative_filepath = "".concat(filepath_stripped, ".").concat(platform_extension, ".").concat(extension);
            var is_relative = alternative_filepath.startsWith('.');
            var absolute_filepath = is_relative
                ? path_1.default.resolve(basedir, alternative_filepath)
                : alternative_filepath;
            try {
                return require.resolve(absolute_filepath, { paths: [basedir] });
            }
            catch (e) {
                // noop
            }
        }
    }
    return null;
};
var resolve = function (filepath, options) {
    if (is_already_specific(filepath, options)) {
        return filepath;
    }
    if (is_module_import(filepath, options)) {
        var resolved_path = require.resolve(filepath, {
            paths: [options.basedir],
        });
        return try_resolve(resolved_path, options);
    }
    var file_resolution = try_resolve(filepath, options);
    if (file_resolution) {
        return file_resolution;
    }
    var index_file = "".concat(filepath, "/index");
    var index_resolution = try_resolve(index_file, options);
    if (index_resolution) {
        return index_resolution;
    }
    return null;
};
exports.resolve = resolve;
exports.default = resolve;
